#ifndef _DOM_RGB_SCREEN_
#define _DOM_RGB_SCREEN_

#include "Arduino.h"

#include <inttypes.h>


#define pinADD_A 3
#define pinADD_B 4
#define pinADD_C 5
#define pinADD_D 16
#define pinLAT 1

#define pinCLK 0
#define PP_CLK_C GPIOB_PCOR
#define PP_CLK_S GPIOB_PSOR
#define PB_CLK (1 << 16)

#define PP_PORT_L_C GPIOD_PCOR
#define PP_PORT_L_S GPIOD_PSOR
#define pinR 2
#define PB_R (1 << 0)

#define pinG 14
#define PB_G (1 << 1)

#define pinB 7
#define PB_B (1 << 2)

#define pinR1 8
#define PB_R1 (1 << 3)

#define pinG1 20
#define PB_G1 (1 << 5)

#define pinB1 21
#define PB_B1 (1 << 6)

#define PP_PORT_H_C GPIOC_PCOR
#define PP_PORT_H_S GPIOC_PSOR
#define pinR2 15
#define PB_R2 (1 << 0)

#define pinG2 12
#define PB_G2 (1 << 7)

#define pinB2 23
#define PB_B2 (1 << 2)

#define pinR3 9
#define PB_R3 (1 << 3)

#define pinG3 10
#define PB_G3 (1 << 4)

#define pinB3 11
#define PB_B3 (1 << 6)


#define SCR_WIDTH 96
#define SCR_HEIGHT 64

// screen refresh rate
#define SCR_FREQ 60
// no of planes i.e. number of bits to process in colours after gamma correction
#define MAX_PLANES 8
// no of rows in the display 8 for 16x32, 16 for 32x32
#define MAX_ROWS 16
// ammount of time to allow for the interrupt routine to shift in a row this is "dead time" between displays 
#define PLANE_ROW_OVERHEAD 30E-6
//number of ticks of "dead time" to add or subtract (to account for LED turn off / on delay)
#define DEAD_TICKS 10

#define PLANE_MASK_MAX_R (1<<(MAX_PLANES-1))
#define PLANE_MASK_MAX_G (1<<(MAX_PLANES-1))<< 8
#define PLANE_MASK_MAX_B (1<<(MAX_PLANES-1))<< 16

/* clock registers CLOCK_SC_* is the main row/plane clock CLOCK_OE_* is the output enable PWM clock */
#define CLOCK_SC TC0
#define CLOCK_SC_NO 0
#define CLOCK_SC_IRQ TC0_IRQn
#define CLOCK_SC_CHAN 0
#define CLOCK_SC_HANDLER TC0_Handler


class DomRGBScreen {
  public:
    DomRGBScreen(int i);
    ~DomRGBScreen();
    int getWidth() { return SCR_WIDTH; };
    int getHeight() { return SCR_HEIGHT; };
    void flip(int bank);
    unsigned char *getScreenMemBase() { return scr_base; };
    unsigned char *getVidMemBase() { return vid_base; };
    void waitVSync();
    void waitLSync();
    void begin();
  protected:
    void updateDisplay(void);
    friend void ftm0_isr(void);
  private:
    unsigned char scr_a[SCR_WIDTH * SCR_HEIGHT * 4];
    unsigned char scr_b[SCR_WIDTH * SCR_HEIGHT * 4];
    unsigned char *scr_base;
    unsigned char *vid_base;
    /* plane_ticks is the number of OE clock ticks to show the plane's row for in CLOCK_OE */
    int plane_ticks[MAX_PLANES];
    volatile int frame=0;
};

extern DomRGBScreen _domRGBScreen;

#endif
